/*
 * (C) Copyright 2015 Nuxeo SA (http://nuxeo.com/) and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * Contributors:
 *     Thomas Roger
 */

package org.nuxeo.ecm.core.io.marshallers.json.document;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import javax.inject.Inject;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParser;
import org.junit.Before;
import org.junit.Test;
import org.nuxeo.ecm.core.api.CoreSession;
import org.nuxeo.ecm.core.api.DocumentModel;
import org.nuxeo.ecm.core.api.impl.DocumentModelImpl;
import org.nuxeo.ecm.core.api.impl.SimpleDocumentModel;
import org.nuxeo.ecm.core.io.marshallers.json.AbstractJsonWriterTest;
import org.nuxeo.ecm.core.io.marshallers.json.JsonAssert;
import org.nuxeo.ecm.core.io.marshallers.json.JsonFactoryProvider;
import org.nuxeo.ecm.core.io.registry.context.RenderingContext;
import org.nuxeo.ecm.core.io.registry.context.RenderingContext.CtxBuilder;

public class DocumentModelJsonReaderTest extends AbstractJsonWriterTest.Local<DocumentModelJsonWriter, DocumentModel> {

    public DocumentModelJsonReaderTest() {
        super(DocumentModelJsonWriter.class, DocumentModel.class);
    }

    private DocumentModel document;

    @Inject
    private CoreSession session;

    @Before
    public void setup() {
        document = session.createDocumentModel("/", "myNote", "Note");
        document.setPropertyValue("dc:title", "My Note");
        document = session.createDocument(document);
    }

    @Test
    public void testDefault() throws Exception {
        RenderingContext renderingContext = CtxBuilder.get();
        renderingContext.setExistingSession(session);
        DocumentModelJsonReader reader = registry.getInstance(renderingContext, DocumentModelJsonReader.class);
        JsonAssert json = jsonAssert(document);
        DocumentModel doc = reader.read(json.getNode());
        assertTrue(doc instanceof DocumentModelImpl);
        assertEquals("myNote", doc.getName());
        assertEquals("My Note", doc.getPropertyValue("dc:title"));
    }

    @Test
    public void testReadSchemaWithoutPrefix() throws IOException {
        String noteJson = "{ \"entity-type\": \"document\",\n" + //
                "  \"type\": \"Note\",\n" + //
                "  \"name\": \"aNote\",\n" + //
                "  \"properties\":\n" + //
                "   { \"dc:title\": \"A note\",\n" + //
                "     \"note:note\": \"note content\" } }";

        DocumentModelJsonReader reader = registry.getInstance(CtxBuilder.get(), DocumentModelJsonReader.class);
        JsonParser jp = JsonFactoryProvider.get().createJsonParser(noteJson);
        JsonNode jn = jp.readValueAsTree();
        DocumentModel noteDocument = reader.read(jn);
        assertNotNull(noteDocument);
        assertTrue(noteDocument instanceof SimpleDocumentModel);
        assertEquals("note content", noteDocument.getPropertyValue("note:note"));
    }

}
