/*
 * (C) Copyright 2010-2015 Nuxeo SA (http://nuxeo.com/) and contributors.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl-2.1.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * Contributors:
 *     Nuxeo
 */
package org.nuxeo.ecm.directory.sql;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.nuxeo.ecm.core.test.annotations.Granularity;
import org.nuxeo.ecm.core.test.annotations.RepositoryConfig;
import org.nuxeo.ecm.directory.Session;
import org.nuxeo.ecm.directory.api.DirectoryService;
import org.nuxeo.runtime.test.runner.Features;
import org.nuxeo.runtime.test.runner.FeaturesRunner;
import org.nuxeo.runtime.test.runner.LocalDeploy;

@RunWith(FeaturesRunner.class)
@Features(SQLDirectoryFeature.class)
@RepositoryConfig(cleanup = Granularity.METHOD)
@LocalDeploy("org.nuxeo.ecm.directory.sql.tests:filterDirectoryContrib.xml")
public class TestFilterDirectories {

    @Inject
    protected DirectoryService directoryService;

    @Test
    public void testFilterDirectory() throws Exception {
        Session unfiltredSession = directoryService.open("unfiltredTestDirectory");
        assertNotNull(unfiltredSession);
        assertEquals(5, unfiltredSession.getEntries().size());
        assertNotNull(unfiltredSession.getEntry("1"));
        assertNotNull(unfiltredSession.getEntry("2"));
        assertNotNull(unfiltredSession.getEntry("5"));

        Map<String, Serializable> queryFilter = new HashMap<String, Serializable>();
        queryFilter.put("lang", "en");
        assertEquals(2, unfiltredSession.query(queryFilter).size());

        Session filtredSession = directoryService.open("filtredTestDirectory");
        assertNotNull(filtredSession);
        assertEquals(2, filtredSession.getEntries().size());
        assertNotNull(filtredSession.getEntry("1"));
        assertNull(filtredSession.getEntry("2"));
        assertNull(filtredSession.getEntry("5"));
        assertEquals(1, filtredSession.query(queryFilter).size());
    }

}
